#Final Project for CHE210D

#import python modules
import numpy as np
import time

#import compiled fortran library
import Qlib4

import sys
import os
import atomwrite


#whether or not to use the 3d visualization
UseVisual = True
if UseVisual:
    #import custom visualization library
    import atomvis


#========== GLOBAL VARIABLES ==========
#distance cutoff for pairwise interactions
Cut = 2.2
#set the random number seed;
np.random.seed()

#NOTE:
#everything below assumes unit atomic masses,
#such that forces = accelerations.

def CalcEnergyForces(Pos, Forces):
    """Wrapper for energy and force calculation.
Input:
    Pos: (N,3) position array
    Forces: (N,3) force array
Output:
    PEnergy: potential energy, float
    Pos: (N,3) position array
    Forces: (N,3) force array (updated)
"""
    global Cut
    return Qlib4.calcenergyforces(L, Cut, Pos, Forces, N1, N2, N3)

def InitPositions(xL, yL, zL, Lat):
    """Returns an array of initial positions of each atom,
placed on a fcc lattice for convenience.
Input:
    Lat: lattice length
    xL, yL, zL: box length
Output:
    Pos: (N,3) array of positions
    i: N atoms (got N and then set array length of Pos) 
"""
    #make the position array
    Pos = np.zeros((912,3), float)

    #compute integer grid # of locations for cubic lattice
    xNLat = 16.
    yNLat = 6.
    zNLat = 19.
    LatSpac = Lat/2.
    #make an array of lattice sites
    xr = LatSpac * np.arange(xNLat, dtype=float)-0.5*xL
    yr = LatSpac * np.arange(yNLat, dtype=float)-0.5*yL
    zr = LatSpac * np.arange(zNLat, dtype=float)

    #loop through x, y, z positions in lattice until done
    #for every atom in the system
    i = 0
    for z in zr:
        for x in xr:
            for y in yr:
                if z in [0.* LatSpac, 2.* LatSpac, 4.* LatSpac, 6.* LatSpac, 8.* LatSpac, 10.* LatSpac, 12.* LatSpac, 14.* LatSpac, 16.* LatSpac, 18.*LatSpac] :
                    x0 = float(int((x + 0.001 + 0.5*xL)/LatSpac))
                    y0 = float(int((y + 0.001 + 0.5*yL)/LatSpac)) 
                    if x0 in [0.,2.,4.,6.,8.,10.,12.,14.] and y0 in [0.,2.,4.,6.]:
                        Pos[i] = np.array([x,y,z], float)
                        i += 1
                    elif x0 in [1.,3.,5.,7.,9.,11.,13.,15.] and y0 in [1., 3., 5., 7.]:
                        Pos[i] = np.array([x,y,z], float)
                        i += 1
                if z in [1.* LatSpac, 3.* LatSpac, 5.* LatSpac, 7.* LatSpac, 9.* LatSpac, 11.* LatSpac, 13.* LatSpac, 15.* LatSpac, 17.*LatSpac] :
                    x0 = float(int((x + 0.001 + 0.5*xL)/LatSpac))
                    y0 = float(int((y + 0.001 + 0.5*yL)/LatSpac)) 
                    if x0 in [0.,2.,4.,6.,8.,10.,12.,14.] and y0 in [1.,3.,5.,7.]:
                        Pos[i] = np.array([x,y,z], float)
                        i += 1
                    elif x0 in [1.,3.,5.,7.,9.,11.,13.,15.] and y0 in [0.,2.,4.,6.]:
                        Pos[i] = np.array([x,y,z], float)
                        i += 1

    return Pos, i

def RescaleVelocities(Vel, T, N1, N2):
    """Rescales velocities in the system to the target temperature.
Input:
    Vel: (N,3) array of atomic velocities
    T: target temperature
    N1, N2: # of atoms of upper wall and fluid
Output:
    Vel: same as above
"""
    #recenter to zero net momentum (assuming all masses same)
    Vel[N1:N1+N2,:] = Vel[N1:N1+N2,:] - Vel[N1:N1+N2,:].mean(axis=0)
    #find the total kinetic energy
    KE = 0.5 * np.sum(Vel[N1:N1+N2,:] * Vel[N1:N1+N2,:])
    #find velocity scale factor from ratios of kinetic energy
    VScale = np.sqrt(1.5 * len(Vel[N1:N1+N2,:]) * T / KE)
    Vel[N1:N1+N2,:] = Vel[N1:N1+N2,:] * VScale
    return Vel  


def InitVelocities(N1, N2, N3, T):
    """Returns an initial random velocity set.
Input:
    N1, N2, N3: # of atoms of upper wall, fluid and lower wall
    T: target temperature
Output:
    Vel: (N,3) array of atomic velocities
"""
    Vel = np.zeros((N1+N2+N3, 3), dtype=float)
    c = np.array([1.,0.,0.], float)
    d = np.array([-1,0.,0.], float)
    Vel[:N1,:] = Vel[:N1,:] + c
    Vel[N1+N2:,:] = Vel[N1+N2:,:] + d
    i = N1
    while i >= N1 :
        Vel[i,:] = np.random.normal(0,((3.3)**0.5)/3, size=3)
        i +=1
        if i >= N1+N2:
            Vel = RescaleVelocities(Vel, T, N1, N2)
            return Vel
    return Vel


def InitAccel(Pos):
    """Returns the initial acceleration array.
Input:
    Pos: (N,3) array of atomic positions
Output:
    PEnergy: potential energy, float
    Pos: (N,3) position array
    Accel: (N,3) array of acceleration vectors
"""
    #get the acceleration from the forces
    PEnergy, Pos,  Accel = CalcEnergyForces(Pos, np.zeros_like(Pos))
    return PEnergy, Pos, Accel

def InstTemp(Vel, N1, N2):
    """Returns the instantaneous temperature.
Input:
    Vel: (N,3) array of atomic velocities
    N1, N2: # of atoms of upper wall and fluid
Output:
    Tinst: float
"""
    return np.sum(Vel[N1:N1+N2,:] * Vel[N1:N1+N2,:]) / (3. * len(Vel[N1:N1+N2,:]))


#### YOUR CODE HERE ####

rho = 0.81
Lat = (4./ rho)**(1./3.)

xL = 8.* Lat
yL = 3.* Lat
zL = 9.* Lat
L = np.array([xL,yL,0.],float)
Cut = 2.2
Temp = 1.1

dt = 0.005
bin = (9./21.)*Lat
MaxSteps = (300./dt)
PdbFreq = 5

Pos, N = InitPositions(xL, yL, zL, Lat)
N1 = int(N/19.*2.)
N3 = int(N/19.*2.)
N2 = int(N/19.*15.)

Names = ['C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C',
         'C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H','H',
         'C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C',
         'C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C','C']
Pdb = atomwrite.pdbfile("movie.pdb", AtomNames = Names)

Vel = InitVelocities(N1, N2, N3, Temp)
PEnergy, Pos, Accel = InitAccel(Pos)


k = 21
Vm = [[] for ki in range(k)]
Velx = np.zeros(21, float)
particle = []
i = 0

while i < MaxSteps:

    Pos, Vel, Accel, KEnergy, PEnergy = Qlib4.vvintegrate(Pos, Vel, Accel, L, Cut, dt, N1, N2, N3)
    if UseVisual:
        if atomvis.Initialized:
            #update the positions
            atomvis.Update(Pos)
        else:
            #initialize the visualization window
            atomvis.Init(Pos)
    if i < (250./dt):
        if i % PdbFreq == 0:
            Pdb.write(Pos)
    i +=1
    Vel = RescaleVelocities(Vel, Temp, N1, N2)

    if i >(250./dt):
        if i % 2 == 0:            
            j = N1
            P= np.zeros(21, dtype=int)
            print PEnergy
            while j < N1+N2:
                m = int(Pos[j,2]/bin)
                if m in [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20]:
                    Velx[m] = Vel[j,0]
                    Vm[m].append(Velx[m])
                    P[m] += 1
                j += 1
            particle.append([P[0],P[1],P[2],P[3],P[4],P[5],P[6],P[7],P[8],P[9],P[10],P[11],P[12],P[13],P[14],P[15],P[16],P[17],P[18],P[19],P[20]])
          
Pdb.close()                
particle = np.array(particle, float)
print np.mean(particle, axis=0)
for m in [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20]: 
    Vm[m] = np.array(Vm[m], float)
    print np.mean(Vm[m]), np.std(Vm[m])

